"use strict";

import { NativeModules, NativeEventEmitter, Platform } from 'react-native';
import { createRef } from 'react';

// @ts-expect-error
const isTurboModuleEnabled = global.__turboModuleProxy != null;
const FileViewerTurbo = isTurboModuleEnabled ? require('./NativeFileViewerTurbo').default : NativeModules.FileViewerTurbo;
const eventEmitter =
// android relies on legacy emitter for backward compatibility
isTurboModuleEnabled && Platform.OS === 'ios' ? null : new NativeEventEmitter(FileViewerTurbo);
const dismissListener = /*#__PURE__*/createRef();
export async function open(path, options = {}) {
  const {
    onDismiss,
    ...nativeOptions
  } = options;
  try {
    await FileViewerTurbo.open(normalize(path), nativeOptions);
    dismissListener.current = addListener('onViewerDidDismiss', () => {
      dismissListener.current?.remove();
      onDismiss && onDismiss();
    });
  } catch (error) {
    throw error;
  }
}
const addListener = (event, listener) => {
  // android relies on legacy emitter for backward compatibility
  return !isTurboModuleEnabled || Platform.OS === 'android' ? eventEmitter?.addListener(event, listener) : FileViewerTurbo[event](listener);
};
function normalize(path) {
  const filePrefix = 'file://';
  if (path.startsWith(filePrefix)) {
    path = path.substring(filePrefix.length);
    try {
      path = decodeURI(path);
    } catch (e) {}
  }
  return path;
}
//# sourceMappingURL=index.js.map